using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace IronPython.Console
{
    public partial class GUIConsole : TextBox
    {
        private Driver _driver;
        private int _currentEditingStart;
        private string _prompt;

        public GUIConsole()
        {
            InitializeComponent();

            Multiline = true;
            AcceptsReturn = true;
            AcceptsTab = true;
            this.KeyDown += new KeyEventHandler(GUIConsole_KeyDown);
            this.KeyPress += new KeyPressEventHandler(GUIConsole_KeyPress);

            _driver = new Driver();
            _driver.CursorMoved += new Driver.CursorMovedEvent(_driver_CursorMoved);
            _driver.TextChanged += new Driver.TextChangedEvent(_driver_TextChanged);
            _driver.Response += new Driver.ResponseEvent(_driver_Response);
            _driver.Prompt += new Driver.ResponseEvent(_driver_Prompt);
            _driver.Start();
        }

        void _driver_Prompt(Driver sender, Driver.ResponseEventArgs args)
        {
            this.AppendText(System.Environment.NewLine);
            this.AppendText(args.Response);
            _prompt = args.Response;
            _currentEditingStart = this.Text.Length;
            this.SelectionStart = this.Text.Length;
        }

        void _driver_Response(Driver sender, Driver.ResponseEventArgs args)
        {
            this.SelectionStart = this.Text.Length;

            switch (args.Style)
            {
                case Driver.ResponseStyle.Output:
                case Driver.ResponseStyle.Error:
                    
                    this.AppendText (System.Environment.NewLine);
                    this.AppendText(args.Response);
                    break;
            }

            this.SelectionStart = this.Text.Length;
        }

        void _driver_TextChanged(Driver sender, Driver.TextChangedEventArgs args)
        {
            /// change the current line
            this.SelectionStart = _currentEditingStart;
            this.SelectionLength = this.Text.Length;
            this.SelectedText = args.NewText;

            /// put the cursor in the appropriate position
            this.SelectionLength = 0;
            this.SelectionStart = _currentEditingStart + args.NewOffset;
        }

        void _driver_CursorMoved(Driver sender, Driver.CursorMovedEventArgs args)
        {
            this.SelectionStart = _currentEditingStart + args.NewOffset;
        }

        void GUIConsole_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
            _driver.CharKeyPressed(e.KeyChar);
        }

        private void GUIConsole_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Left:
                    e.Handled = true;
                    e.SuppressKeyPress = true;
                    _driver.LeftPressed();
                    break;

                case Keys.Right:
                    e.Handled = true;
                    e.SuppressKeyPress = true;
                    _driver.RightPressed();
                    break;

                case Keys.Back:
                    e.Handled = true;
                    e.SuppressKeyPress = true;
                    _driver.BackspacePressed();
                    break;

                case Keys.Enter:
                    e.Handled = true;
                    e.SuppressKeyPress = true;
                    _driver.EnterPressed();
                    break;

                case Keys.Up:
                    e.Handled = true;
                    e.SuppressKeyPress = true;
                    _driver.UpPressed();
                    break;

                case Keys.Down:
                    e.Handled = true;
                    e.SuppressKeyPress = true;
                    _driver.DownPressed();
                    break;

                case Keys.Delete:
                    e.Handled = true;
                    e.SuppressKeyPress = true;
                    _driver.DelPressed ();
                    break;
            }
        }

        private void InitializeComponent()
        {
            this.SuspendLayout();
            // 
            // GUIConsole
            // 
            this.Font = new System.Drawing.Font("Courier New", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(177)));
            this.ResumeLayout(false);

        }
    }
}

